@@ -1789,7 +1789,7 @@ void ical_putics(void)
 				HashPos = GetNewHashPos(tzidlist, 0);
 
 				while (GetNextHashPos(tzidlist, HashPos, &len, &Key, &Value)) {
-					CtdlLogPrintf(CTDL_DEBUG, "Attaching timezone '%s'\n", Value);
+					CtdlLogPrintf(CTDL_DEBUG, "Attaching timezone '%s'\n", (char*) Value);
 					icaltimezone *t = NULL;
 
 					/* First look for a timezone attached to the original calendar */
@@ -2344,7 +2344,7 @@ void ical_obj_beforesave_backend(char *name, char *filename, char *partnum,
 					if (msg->cm_fields['U'] != NULL) {
 						free(msg->cm_fields['U']);
 					}
-					msg->cm_fields['U'] = strdup(buf);
+					msg->cm_fields['U'] = rfc2047encode(buf, strlen(buf));
 				}
 			}
 
