@@ -1,5 +1,5 @@
 /*
- * $Id: serv_imap.c 8686 2010-07-19 20:56:54Z dothebart $ 
+ * $Id$ 
  *
  * IMAP server for the Citadel system
  * Copyright (C) 2000-2009 by Art Cancro and others.
@@ -128,7 +128,7 @@ void imap_free_transmitted_message(void)
  */
 void imap_set_seen_flags(int first_msg)
 {
-	struct visit vbuf;
+	visit vbuf;
 	int i;
 	int num_sets;
 	int s;
@@ -1744,5 +1744,5 @@ CTDL_MODULE_INIT(imap)
 	}
 	
 	/* return our Subversion id for the Log */
-	return "$Id: serv_imap.c 8686 2010-07-19 20:56:54Z dothebart $";
+	return "$Id$";
 }
