@@ -1,11 +1,9 @@
 /*
- * $Id$
- *
  * This module dumps and/or loads the Citadel database in XML format.
  *
- * Copyright (c) 1987-2010 by the citadel.org team
+ * Copyright (c) 1987-2011 by the citadel.org team
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is open source software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
@@ -86,8 +84,11 @@ void xml_strout(char *str) {
 	char *c = str;
 
 	while (*c != 0) {
-		if (*c == '&') {
-			client_write("&amp;", 5);
+		if (*c == '\"') {
+			client_write("&quot;", 6);
+		}
+		else if (*c == '\'') {
+			client_write("&apos;", 6);
 		}
 		else if (*c == '<') {
 			client_write("&lt;", 4);
@@ -95,6 +96,9 @@ void xml_strout(char *str) {
 		else if (*c == '>') {
 			client_write("&gt;", 4);
 		}
+		else if (*c == '&') {
+			client_write("&amp;", 5);
+		}
 		else {
 			client_write(c, 1);
 		}
@@ -142,7 +146,11 @@ void migr_export_rooms_backend(struct ctdlroom *buf, void *data) {
 	cprintf("<QRhighest>%ld</QRhighest>\n", buf->QRhighest);
 	cprintf("<QRgen>%ld</QRgen>\n", (long)buf->QRgen);
 	cprintf("<QRflags>%u</QRflags>\n", buf->QRflags);
-	client_write("<QRdirname>", 11);	xml_strout(buf->QRdirname);	client_write("</QRdirname>\n", 13);
+	if (buf->QRflags & QR_DIRECTORY) {
+		client_write("<QRdirname>", 11);
+		xml_strout(buf->QRdirname);
+		client_write("</QRdirname>\n", 13);
+	}
 	cprintf("<QRinfo>%ld</QRinfo>\n", buf->QRinfo);
 	cprintf("<QRfloor>%d</QRfloor>\n", buf->QRfloor);
 	cprintf("<QRmtime>%ld</QRmtime>\n", (long)buf->QRmtime);
@@ -182,7 +190,7 @@ void migr_export_rooms(void) {
 	 * this will be handled by exporting the reference count, not by
 	 * exporting the message multiple times.)
 	 */
-	snprintf(cmd, sizeof cmd, "sort <%s >%s", migr_tempfilename1, migr_tempfilename2);
+	snprintf(cmd, sizeof cmd, "sort -n <%s >%s", migr_tempfilename1, migr_tempfilename2);
 	if (system(cmd) != 0) CtdlLogPrintf(CTDL_ALERT, "Error %d\n", errno);
 	snprintf(cmd, sizeof cmd, "uniq <%s >%s", migr_tempfilename2, migr_tempfilename1);
 	if (system(cmd) != 0) CtdlLogPrintf(CTDL_ALERT, "Error %d\n", errno);
@@ -208,6 +216,23 @@ void migr_export_floors(void) {
 }
 
 
+/*
+ * Return nonzero if the supplied string contains only characters which are valid in a sequence set.
+ */
+int is_sequence_set(char *s) {
+	if (!s) return(0);
+
+	char *c = s;
+	char ch;
+	while (ch = *c++, ch) {
+		if (!strchr("0123456789*,:", ch)) {
+			return(0);
+		}
+	}
+	return(1);
+}
+
+
 
 /* 
  *  Traverse the visits file...
@@ -231,7 +256,7 @@ void migr_export_visits(void) {
 		cprintf("<v_usernum>%ld</v_usernum>\n", vbuf.v_usernum);
 
 		client_write("<v_seen>", 8);
-		if (!IsEmptyStr(vbuf.v_seen)) {
+		if ( (!IsEmptyStr(vbuf.v_seen)) && (is_sequence_set(vbuf.v_seen)) ) {
 			xml_strout(vbuf.v_seen);
 		}
 		else {
@@ -239,7 +264,12 @@ void migr_export_visits(void) {
 		}
 		client_write("</v_seen>", 9);
 
-		client_write("<v_answered>", 12); xml_strout(vbuf.v_answered); client_write("</v_answered>\n", 14);
+		if ( (!IsEmptyStr(vbuf.v_answered)) && (is_sequence_set(vbuf.v_answered)) ) {
+			client_write("<v_answered>", 12);
+			xml_strout(vbuf.v_answered);
+			client_write("</v_answered>\n", 14);
+		}
+
 		cprintf("<v_flags>%u</v_flags>\n", vbuf.v_flags);
 		cprintf("<v_view>%d</v_view>\n", vbuf.v_view);
 		client_write("</visit>\n", 9);
@@ -549,7 +579,7 @@ void migr_xml_start(void *data, const char *el, const char **attr) {
 	}
 
 	if (citadel_migrate_data != 1) {
-		CtdlLogPrintf(CTDL_ALERT, "Out-of-sequence tag <%s> detected.  Warning: ODD-DATA!\n");
+		CtdlLogPrintf(CTDL_ALERT, "Out-of-sequence tag <%s> detected.  Warning: ODD-DATA!\n", el);
 		return;
 	}
 
@@ -594,7 +624,7 @@ void migr_xml_end(void *data, const char *el) {
 	}
 
 	if (citadel_migrate_data != 1) {
-		CtdlLogPrintf(CTDL_ALERT, "Out-of-sequence tag <%s> detected.  Warning: ODD-DATA!\n");
+		CtdlLogPrintf(CTDL_ALERT, "Out-of-sequence tag <%s> detected.  Warning: ODD-DATA!\n", el);
 		return;
 	}
 
