@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * This module is an SMTP and ESMTP implementation for the Citadel system.
  * It is compliant with all of the following:
  *
@@ -22,27 +20,28 @@
  * The VRFY and EXPN commands have been removed from this implementation
  * because nobody uses these commands anymore, except for spammers.
  *
- * Copyright (c) 1998-2009 by the citadel.org team
+ * Copyright (c) 1998-2011 by the citadel.org team
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
- *  (at your option) any later version.
+ * This program is open source software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include "sysdep.h"
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
+#include <termios.h>
 #include <fcntl.h>
 #include <signal.h>
 #include <pwd.h>
@@ -1035,7 +1034,7 @@ void smtp_try(const char *key, const char *addr, int *status,
 				}
 				if ( (lp>=0) && (rp>lp) ) {
 					mailfrom[rp] = 0;
-					strcpy(mailfrom, &mailfrom[lp]);
+					strcpy(mailfrom, &mailfrom[lp + 1]);
 				}
 	
 				scan_done = 1;
@@ -1085,7 +1084,21 @@ void smtp_try(const char *key, const char *addr, int *status,
 		CtdlLogPrintf(CTDL_DEBUG, "SMTP client: connecting to %s : %s ...\n", mx_host, mx_port);
 		sock = sock_connect(mx_host, mx_port, "tcp");
 		snprintf(dsn, SIZ, "Could not connect: %s", strerror(errno));
-		if (sock >= 0) CtdlLogPrintf(CTDL_DEBUG, "SMTP client: connected!\n");
+		if (sock >= 0) 
+		{
+			CtdlLogPrintf(CTDL_DEBUG, "SMTP client: connected!\n");
+				int fdflags; 
+				fdflags = fcntl(sock, F_GETFL);
+				if (fdflags < 0)
+					CtdlLogPrintf(CTDL_DEBUG,
+						      "unable to get SMTP-Client socket flags! %s \n",
+						      strerror(errno));
+				fdflags = fdflags | O_NONBLOCK;
+				if (fcntl(sock, F_SETFL, fdflags) < 0)
+					CtdlLogPrintf(CTDL_DEBUG,
+						      "unable to set SMTP-Client socket nonblocking flags! %s \n",
+						      strerror(errno));
+		}
 		if (sock < 0) {
 			if (errno > 0) {
 				snprintf(dsn, SIZ, "%s", strerror(errno));
@@ -1106,7 +1119,7 @@ void smtp_try(const char *key, const char *addr, int *status,
 	CCC->sPos = NULL;
 
 	/* Process the SMTP greeting from the server */
-	if (ml_sock_gets(&sock, buf) < 0) {
+	if (ml_sock_gets(&sock, buf, 90) < 0) {
 		*status = 4;
 		strcpy(dsn, "Connection broken during SMTP conversation");
 		goto bail;
@@ -1131,7 +1144,7 @@ void smtp_try(const char *key, const char *addr, int *status,
 	snprintf(buf, sizeof buf, "EHLO %s\r\n", config.c_fqdn);
 	CtdlLogPrintf(CTDL_DEBUG, ">%s", buf);
 	sock_write(&sock, buf, strlen(buf));
-	if (ml_sock_gets(&sock, buf) < 0) {
+	if (ml_sock_gets(&sock, buf, 30) < 0) {
 		*status = 4;
 		strcpy(dsn, "Connection broken during SMTP HELO");
 		goto bail;
@@ -1141,7 +1154,7 @@ void smtp_try(const char *key, const char *addr, int *status,
 		snprintf(buf, sizeof buf, "HELO %s\r\n", config.c_fqdn);
 		CtdlLogPrintf(CTDL_DEBUG, ">%s", buf);
 		sock_write(&sock, buf, strlen(buf));
-		if (ml_sock_gets(&sock, buf) < 0) {
+		if (ml_sock_gets(&sock, buf, 30) < 0) {
 			*status = 4;
 			strcpy(dsn, "Connection broken during SMTP HELO");
 			goto bail;
@@ -1168,7 +1181,7 @@ void smtp_try(const char *key, const char *addr, int *status,
 		snprintf(buf, sizeof buf, "AUTH PLAIN %s\r\n", encoded);
 		CtdlLogPrintf(CTDL_DEBUG, ">%s", buf);
 		sock_write(&sock, buf, strlen(buf));
-		if (ml_sock_gets(&sock, buf) < 0) {
+		if (ml_sock_gets(&sock, buf, 30) < 0) {
 			*status = 4;
 			strcpy(dsn, "Connection broken during SMTP AUTH");
 			goto bail;
@@ -1192,7 +1205,7 @@ void smtp_try(const char *key, const char *addr, int *status,
 	snprintf(buf, sizeof buf, "MAIL FROM:<%s>\r\n", envelope_from);
 	CtdlLogPrintf(CTDL_DEBUG, ">%s", buf);
 	sock_write(&sock, buf, strlen(buf));
-	if (ml_sock_gets(&sock, buf) < 0) {
+	if (ml_sock_gets(&sock, buf, 30) < 0) {
 		*status = 4;
 		strcpy(dsn, "Connection broken during SMTP MAIL");
 		goto bail;
@@ -1215,7 +1228,7 @@ void smtp_try(const char *key, const char *addr, int *status,
 	snprintf(buf, sizeof buf, "RCPT TO:<%s@%s>\r\n", user, node);
 	CtdlLogPrintf(CTDL_DEBUG, ">%s", buf);
 	sock_write(&sock, buf, strlen(buf));
-	if (ml_sock_gets(&sock, buf) < 0) {
+	if (ml_sock_gets(&sock, buf, 30) < 0) {
 		*status = 4;
 		strcpy(dsn, "Connection broken during SMTP RCPT");
 		goto bail;
@@ -1237,7 +1250,7 @@ void smtp_try(const char *key, const char *addr, int *status,
 	/* RCPT succeeded, now try the DATA command */
 	CtdlLogPrintf(CTDL_DEBUG, ">DATA\n");
 	sock_write(&sock, "DATA\r\n", 6);
-	if (ml_sock_gets(&sock, buf) < 0) {
+	if (ml_sock_gets(&sock, buf, 30) < 0) {
 		*status = 4;
 		strcpy(dsn, "Connection broken during SMTP DATA");
 		goto bail;
@@ -1257,7 +1270,10 @@ void smtp_try(const char *key, const char *addr, int *status,
 	}
 
 	/* If we reach this point, the server is expecting data.*/
-	sock_write(&sock, msgtext, msg_size);
+	sock_write_timeout(&sock, 
+			   msgtext, 
+			   msg_size, 
+			   (msg_size / 128) + 50);
 	if (msgtext[msg_size-1] != 10) {
 		CtdlLogPrintf(CTDL_WARNING, "Possible problem: message did not "
 			"correctly terminate. (expecting 0x10, got 0x%02x)\n",
@@ -1266,7 +1282,8 @@ void smtp_try(const char *key, const char *addr, int *status,
 	}
 
 	sock_write(&sock, ".\r\n", 3);
-	if (ml_sock_gets(&sock, buf) < 0) {
+	tcdrain(sock);
+	if (ml_sock_gets(&sock, buf, 90) < 0) {
 		*status = 4;
 		strcpy(dsn, "Connection broken during SMTP message transmit");
 		goto bail;
@@ -1291,7 +1308,7 @@ void smtp_try(const char *key, const char *addr, int *status,
 
 	CtdlLogPrintf(CTDL_DEBUG, ">QUIT\n");
 	sock_write(&sock, "QUIT\r\n", 6);
-	ml_sock_gets(&sock, buf);
+	ml_sock_gets(&sock, buf, 30);
 	CtdlLogPrintf(CTDL_DEBUG, "<%s\n", buf);
 	CtdlLogPrintf(CTDL_INFO, "SMTP client: delivery to <%s> @ <%s> (%s) succeeded\n",
 		user, node, name);
@@ -1466,7 +1483,7 @@ void smtp_do_bounce(char *instr) {
         StrBufAppendBufPlain(BounceMB, HKEY("--"), 0);
 	StrBufAppendBuf(BounceMB, boundary, 0);
 	StrBufAppendBufPlain(BounceMB, HKEY("--\r\n"), 0);
-	bmsg->cm_fields['A'] = SmashStrBuf(&BounceMB);
+	bmsg->cm_fields['M'] = SmashStrBuf(&BounceMB);
 	/* Deliver the bounce if there's anything worth mentioning */
 	CtdlLogPrintf(CTDL_DEBUG, "num_bounces = %d\n", num_bounces);
 	if (num_bounces > 0) {
@@ -1755,6 +1772,7 @@ void *smtp_do_queue(void *arg) {
 
 	citthread_mutex_unlock (&smtp_send_lock);
 	CtdlLogPrintf(CTDL_INFO, "SMTP client: queue run completed; %d messages processed\n", num_processed);
+	run_queue_now = 0;
 
 	CtdlClearSystemContext();
 	return(NULL);
