@@ -1,5 +1,5 @@
 /* 
- * $Id: room_ops.c 8575 2010-05-18 22:52:46Z dothebart $
+ * $Id$
  * 
  * Server functions which perform operations on room objects.
  *
@@ -53,7 +53,7 @@ void CtdlRoomAccess(struct ctdlroom *roombuf, struct ctdluser *userbuf,
 		int *result, int *view)
 {
 	int retval = 0;
-	struct visit vbuf;
+	visit vbuf;
 
 	/* for internal programs, always do everything */
 	if (((CC->internal_pgm)) && (roombuf->QRflags & QR_INUSE)) {
@@ -887,7 +887,7 @@ void CtdlUserGoto(char *where, int display_result, int transiently,
 	int rmailflag;
 	int raideflag;
 	int newmailcount = 0;
-	struct visit vbuf;
+	visit vbuf;
 	char truncated_roomname[ROOMNAMELEN];
         struct cdbdata *cdbfr;
 	long *msglist = NULL;
@@ -1834,7 +1834,7 @@ unsigned CtdlCreateRoom(char *new_room_name,
 
 	struct ctdlroom qrbuf;
 	struct floor flbuf;
-	struct visit vbuf;
+	visit vbuf;
 
 	CtdlLogPrintf(CTDL_DEBUG, "CtdlCreateRoom(name=%s, type=%d, view=%d)\n",
 		new_room_name, new_room_type, new_room_view);
@@ -2264,5 +2264,5 @@ CTDL_MODULE_INIT(room_ops)
 		CtdlRegisterProtoHook(cmd_eflr, "EFLR", "Autoconverted. TODO: document me.");
 	}
         /* return our Subversion id for the Log */
-	return "$Id: room_ops.c 8575 2010-05-18 22:52:46Z dothebart $";
+	return "$Id$";
 }
