@@ -1,5 +1,5 @@
 /*
- * $Id: sysdep.c 8634 2010-06-20 22:28:28Z dothebart $
+ * $Id$
  *
  * Citadel "system dependent" stuff.
  * See COPYING for copyright information.
@@ -516,8 +516,27 @@ int client_write(const char *buf, int nbytes)
 
 	if (nbytes < 1) return(0);
 
-//	flush_client_inbuf();
 	Ctx = CC;
+
+#ifdef BIGBAD_IODBG
+	{
+		int rv = 0;
+		char fn [SIZ];
+		FILE *fd;
+		
+		snprintf(fn, SIZ, "/tmp/foolog_%s.%d", Ctx->ServiceName, Ctx->cs_pid);
+		
+		fd = fopen(fn, "a+");
+		fprintf(fd, "Sending: BufSize: %d BufContent: [",
+			nbytes);
+		rv = fwrite(buf, nbytes, 1, fd);
+		fprintf(fd, "]\n");
+		
+			
+		fclose(fd);
+	}
+#endif
+//	flush_client_inbuf();
 	if (Ctx->redirect_buffer != NULL) {
 		StrBufAppendBufPlain(Ctx->redirect_buffer,
 				     buf, nbytes, 0);
